@@ -3,7 +3,7 @@
 //
 // MIT License
 //
-// Copyright (c) 2014-2019 Michael Truog <mjtruog at protonmail dot com>
+// Copyright (c) 2014-2020 Michael Truog <mjtruog at protonmail dot com>
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -36,6 +36,14 @@ var fs = require('fs');
 var assert = require('assert');
 
 var toNativeString = {}.toString;
+CloudI.nodejs_version_after = Erlang.nodejs_version_after;
+var bufferFrom;
+if (Erlang.nodejs_version_after('5.10.0',true)) {
+    bufferFrom = Buffer.from;
+}
+else {
+    bufferFrom = Buffer;
+}
 var littleEndian = (
     (new Uint16Array((new Uint8Array([0,1])).buffer))[0] === 0x0100);
 var unpackUint8 = function unpackUint8 (i, data) {
@@ -67,25 +75,12 @@ else {
     unpackUint32 = unpackUint32big;
 }
 var packUint32big = function packUint32big (value) {
-    return new Buffer([(value >>> 24) & 0xff,
-                       (value >>> 16) & 0xff,
-                       (value >>> 8) & 0xff,
-                       value & 0xff]);
+    return new bufferFrom([(value >>> 24) & 0xff,
+                           (value >>> 16) & 0xff,
+                           (value >>> 8) & 0xff,
+                           value & 0xff]);
 };
-var nodejs_version = process.versions['node'].split('.').map(s => parseInt(s));
-var nodejsVersionAfter = function nodejsVersionAfter (s, include) {
-    var v = s.split('.').map(s => parseInt(s));
-    for (var i = 0; i < v.length; i++) {
-        if (nodejs_version[i] > v[i]) {
-            return true;
-        }
-        if (nodejs_version[i] < v[i]) {
-            return false;
-        }
-    }
-    return include;
-};
-if (nodejsVersionAfter('10.0.0',true)) {
+if (Erlang.nodejs_version_after('10.0.0',true)) {
     var originalEmitWarning = process.emitWarning;
     process.emitWarning = function(warning, type, code, ctor) {
         if (code === 'DEP0097') {
@@ -238,29 +233,20 @@ CloudI.API = function API (thread_index, callback) {
     else {
         throw new InvalidInputException();
     }
-    if (nodejsVersionAfter('0.12.1',false)) {
-        API._s_in = new net.Socket({fd: (thread_index + 3),
-                                    readable: true,
-                                    writable: true});
-        API._s_out = API._s_in;
-    }
-    else {
-        API._s_in = new net.Socket({fd: (thread_index + 3),
-                                    readable: true,
-                                    writable: false});
-        API._s_out = fs.createWriteStream(null, {fd: (thread_index + 3)});
-    }
+    API._socket = new net.Socket({fd: (thread_index + 3),
+                                  readable: true,
+                                  writable: true});
     API._initialization_complete = false;
     API._terminate = false;
     API._terminate_callback = undefined;
     API._size = parseInt(buffer_size_str);
     API._callbacks = {};
-    API._timeout_terminate = 1000; // TIMEOUT_TERMINATE_MIN
+    API._timeout_terminate = 10; // TIMEOUT_TERMINATE_MIN
     API._poll_callback = callback;
     API._poll_callbacks_pending = [];
     API._poll_data = undefined;
     API._poll_data_size = undefined;
-    API._s_in.on('data', function(data) {
+    API._socket.on('data', function(data) {
         try {
             if (! API._use_header) {
                 API._poll_request(data);
@@ -294,7 +280,6 @@ CloudI.API = function API (thread_index, callback) {
                 err instanceof MessageDecodingException) {
                 CloudI.stderr_write(err.stack + '\n');
                 API._poll_terminate();
-                process.exit(1);
             }
             else if (err instanceof TerminateException) {
                 API._poll_terminate();
@@ -302,14 +287,12 @@ CloudI.API = function API (thread_index, callback) {
             else {
                 API._exception(err);
                 API._poll_terminate();
-                process.exit(1);
             }
         }
     });
-    API._s_in.on('error', function(err) {
+    API._socket.on('error', function(err) {
         API._exception(err);
         API._poll_terminate();
-        process.exit(1);
     });
     API._send(new Erlang.OtpErlangAtom('init'));
 };
@@ -592,33 +575,33 @@ CloudI.API.prototype._callback = function (command, name, pattern,
             domain_async.name = 'CloudI Async';
             domain_async.on('error', function(err) {
                 domain_async.exit();
-                if (err instanceof InvalidInputException ||
-                    err instanceof MessageDecodingException) {
-                    CloudI.stderr_write(err.stack + '\n');
-                    API._poll_terminate();
-                    process.exit(1);
+                var return_null_response = false;
+                if (err instanceof MessageDecodingException) {
+                    API._terminate = true;
+                    return_null_response = true;
                 }
                 else if (err instanceof TerminateException) {
-                    API._poll_terminate();
+                    return_null_response = true;
                 }
                 else if (err instanceof ReturnAsyncException) {
                     return;
                 }
                 else if (err instanceof ReturnSyncException) {
+                    API._terminate = true;
                     CloudI.stderr_write(err.stack + '\n');
-                    API._poll_terminate();
-                    process.exit(1);
                 }
                 else if (err instanceof ForwardAsyncException) {
                     return;
                 }
                 else if (err instanceof ForwardSyncException) {
+                    API._terminate = true;
                     CloudI.stderr_write(err.stack + '\n');
-                    API._poll_terminate();
-                    process.exit(1);
                 }
                 else {
+                    return_null_response = true;
                     API._exception(err);
+                }
+                if (return_null_response) {
                     var response_info = '';
                     var response = '';
                     try {
@@ -630,6 +613,9 @@ CloudI.API.prototype._callback = function (command, name, pattern,
                         err_new = undefined;
                     }
                 }
+                if (API._terminate) {
+                    API._poll_terminate();
+                }
             });
             domain_async.enter();
             process.nextTick(function () {
@@ -671,33 +657,33 @@ CloudI.API.prototype._callback = function (command, name, pattern,
             domain_sync.name = 'CloudI Sync';
             domain_sync.on('error', function(err) {
                 domain_sync.exit();
-                if (err instanceof InvalidInputException ||
-                    err instanceof MessageDecodingException) {
-                    CloudI.stderr_write(err.stack + '\n');
-                    API._poll_terminate();
-                    process.exit(1);
+                var return_null_response = false;
+                if (err instanceof MessageDecodingException) {
+                    API._terminate = true;
+                    return_null_response = true;
                 }
                 else if (err instanceof TerminateException) {
-                    API._poll_terminate();
+                    return_null_response = true;
                 }
                 else if (err instanceof ReturnAsyncException) {
+                    API._terminate = true;
                     CloudI.stderr_write(err.stack + '\n');
-                    API._poll_terminate();
-                    process.exit(1);
                 }
                 else if (err instanceof ReturnSyncException) {
                     return;
                 }
                 else if (err instanceof ForwardAsyncException) {
+                    API._terminate = true;
                     CloudI.stderr_write(err.stack + '\n');
-                    API._poll_terminate();
-                    process.exit(1);
                 }
                 else if (err instanceof ForwardSyncException) {
                     return;
                 }
                 else {
+                    return_null_response = true;
                     API._exception(err);
+                }
+                if (return_null_response) {
                     var response_info = '';
                     var response = '';
                     try {
@@ -709,6 +695,9 @@ CloudI.API.prototype._callback = function (command, name, pattern,
                         err_new = undefined;
                     }
                 }
+                if (API._terminate) {
+                    API._poll_terminate();
+                }
             });
             domain_sync.enter();
             process.nextTick(function () {
@@ -762,6 +751,7 @@ CloudI.API.prototype._handle_events = function (data, data_size, i, command) {
     while (true) {
         switch (command) {
             case MESSAGE_TERM:
+                API._terminate = true;
                 throw new TerminateException(API._timeout_terminate);
             case MESSAGE_REINIT:
                 API._process_count = unpackUint32(i, data);
@@ -872,7 +862,6 @@ CloudI.API.prototype._poll_request = function (data) {
                     if (err) {
                         API._exception(err);
                         API._poll_terminate();
-                        process.exit(1);
                     }
                     else {
                         API._callback(command, name, pattern,
@@ -1014,10 +1003,11 @@ CloudI.API.prototype._poll_wait = function (f) {
 CloudI.API.prototype._poll_terminate = function () {
     var API = this;
     API._terminate = true;
-    API._s_in.destroy();
+    API._socket.destroy();
     if (API._terminate_callback !== undefined) {
         API._terminate_callback(false);
     }
+    process.exit();
 };
 
 CloudI.API.prototype._poll_blocked = function () {
@@ -1089,29 +1079,58 @@ CloudI.API.prototype.shutdown = function (callback, reason) {
     });
 };
 
-CloudI.API.prototype._text_key_value_parse = function (text) {
-    var result = {};
-    var data = text.toString('binary').split('\x00');
-    for (var i = 0; i < data.length - 1; i += 2) {
-        var key = data[i];
-        var current = result[key];
+// class method
+var text_pairs_parse = function text_pairs_parse (text) {
+    var pairs = {};
+    var text_segments = text.toString('binary').split('\x00');
+    for (var i = 0; i < text_segments.length - 1; i += 2) {
+        var key = text_segments[i];
+        var current = pairs[key];
         if (current === undefined) {
-            result[key] = data[i + 1];
+            pairs[key] = text_segments[i + 1];
         }
         else if (typeof current === 'object' && 
                  toNativeString.call(response) == '[object Array]') {
-            current.push(data[i + 1]);
+            current.push(text_segments[i + 1]);
         }
         else {
-            result[key] = [current, data[i + 1]];
+            pairs[key] = [current, text_segments[i + 1]];
         }
     }
-    return result;
+    return pairs;
 };
 
-CloudI.API.prototype.info_key_value_parse = function (message_info) {
-    var API = this;
-    return API._text_key_value_parse(message_info);
+// class method
+var text_pairs_new = function text_pairs_new (pairs, response) {
+    var text_segments = [];
+    for (var key in pairs) {
+        var values = pairs[key];
+        if (typeof values === 'string') {
+            text_segments.push(key);
+            text_segments.push(values);
+        }
+        else {
+            for (var value in values) {
+                text_segments.push(key);
+                text_segments.push(value);
+            }
+        }
+    }
+    if (response && text_segments.length == 0) {
+        return '\x00';
+    }
+    text_segments.push('');
+    return text_segments.join('\x00');
+};
+
+CloudI.API.info_key_value_parse = function info_key_value_parse (info) {
+    return text_pairs_parse(info);
+};
+
+CloudI.API.info_key_value_new = function info_key_value_new (pairs, response) {
+    response = typeof response !== 'undefined' ?
+               response : true;
+    return text_pairs_new(pairs, response);
 };
 
 CloudI.API.prototype._exception = function (err) {
@@ -1127,13 +1146,11 @@ CloudI.API.prototype._send = function (terms) {
         if (err) {
             API._exception(err);
             API._poll_terminate();
-            process.exit(1);
-            return;
         }
         if (API._use_header) {
             data = Buffer.concat([packUint32big(data.length), data]);
         }
-        API._s_out.write(data, 'binary');
+        API._socket.write(data, 'binary');
     });
 };
 
