@@ -9,7 +9,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2011-2019 Michael Truog <mjtruog at protonmail dot com>
+%%% Copyright (c) 2011-2020 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -30,8 +30,8 @@
 %%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog at protonmail dot com>
-%%% @copyright 2011-2019 Michael Truog
-%%% @version 1.8.0 {@date} {@time}
+%%% @copyright 2011-2020 Michael Truog
+%%% @version 1.8.1 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_core_i_configurator).
@@ -49,6 +49,8 @@
          services_add/2,
          services_remove/2,
          services_restart/2,
+         services_suspend/2,
+         services_resume/2,
          services_update/2,
          services_search/3,
          service_ids/1,
@@ -69,6 +71,8 @@
          service_start/2,
          service_stop/3,
          service_restart/2,
+         service_suspend/2,
+         service_resume/2,
          service_update/2,
          service_update_external/5,
          service_process_init_end/1,
@@ -108,22 +112,34 @@
 -type error_reason_service_restart() ::
     {service_internal_restart_failed |
      service_external_restart_failed, any()}.
+-type error_reason_service_suspend() ::
+    {service_internal_suspend_failed |
+     service_external_suspend_failed, any()}.
+-type error_reason_service_resume() ::
+    {service_internal_resume_failed |
+     service_external_resume_failed, any()}.
 -type error_reason_service_update() ::
     {service_internal_update_failed |
      service_external_update_failed, any()}.
 -type error_reason_services_search() ::
     service_scope_invalid.
+-type error_reason_nodes_set() ::
+    {set_invalid, any()}.
 -type error_reason_code_status() ::
     {file, {file:filename(), file:posix() | badarg}}.
 -export_type([error_reason_service_start/0,
               error_reason_service_stop/0,
               error_reason_service_restart/0,
+              error_reason_service_suspend/0,
+              error_reason_service_resume/0,
               error_reason_service_update/0,
               error_reason_services_search/0,
+              error_reason_nodes_set/0,
               error_reason_code_status/0]).
 
 -record(state,
     {
+        node_name :: nonempty_string(),
         time_start :: cloudi_timestamp:native_monotonic(),
         configuration :: #config{}
     }).
@@ -174,6 +190,14 @@ services_restart(L, Timeout) ->
     ?CATCH_EXIT(gen_server:call(?MODULE,
                                 {services_restart, L}, Timeout)).
 
+services_suspend(L, Timeout) ->
+    ?CATCH_EXIT(gen_server:call(?MODULE,
+                                {services_suspend, L}, Timeout)).
+
+services_resume(L, Timeout) ->
+    ?CATCH_EXIT(gen_server:call(?MODULE,
+                                {services_resume, L}, Timeout)).
+
 services_update(L, Timeout) ->
     ?CATCH_EXIT(gen_server:call(?MODULE,
                                 {services_update, L}, Timeout)).
@@ -308,6 +332,30 @@ service_restart(#config_service_internal{} = Service, Timeout) ->
 service_restart(#config_service_external{} = Service, Timeout) ->
     service_restart_external(Service, timeout_decr(Timeout)).
 
+-spec service_suspend(#config_service_internal{} |
+                      #config_service_external{},
+                      Timeout :: pos_integer() | infinity) ->
+    ok |
+    {error, error_reason_service_suspend()}.
+
+service_suspend(#config_service_internal{} = Service, Timeout) ->
+    service_suspend_internal(Service, timeout_decr(Timeout));
+
+service_suspend(#config_service_external{} = Service, Timeout) ->
+    service_suspend_external(Service, timeout_decr(Timeout)).
+
+-spec service_resume(#config_service_internal{} |
+                     #config_service_external{},
+                     Timeout :: pos_integer() | infinity) ->
+    ok |
+    {error, error_reason_service_resume()}.
+
+service_resume(#config_service_internal{} = Service, Timeout) ->
+    service_resume_internal(Service, timeout_decr(Timeout));
+
+service_resume(#config_service_external{} = Service, Timeout) ->
+    service_resume_external(Service, timeout_decr(Timeout)).
+
 -spec service_update(#config_service_update{},
                      Timeout :: pos_integer() | infinity) ->
     {ok, nonempty_list(cloudi_service_api:service_id())} |
@@ -352,7 +400,10 @@ service_terminated(ID)
 %%%------------------------------------------------------------------------
 
 init([Config]) ->
-    {ok, #state{time_start = cloudi_timestamp:native_monotonic(),
+    TimeStart = cloudi_timestamp:native_monotonic(),
+    NodeName = cloudi_core_i_nodes:node_name(node()),
+    {ok, #state{node_name = NodeName,
+                time_start = TimeStart,
                 configuration = Config}}.
 
 handle_call(configure, _, State) ->
@@ -414,8 +465,26 @@ handle_call({services_remove, L}, _,
 handle_call({services_restart, L}, _,
             #state{configuration = Config} = State) ->
     case cloudi_core_i_configuration:services_restart(L, Config, infinity) of
-        {ok, ConfigNew} ->
-            {reply, ok, State#state{configuration = ConfigNew}};
+        ok ->
+            {reply, ok, State};
+        {error, _} = Error ->
+            {reply, Error, State}
+    end;
+
+handle_call({services_suspend, L}, _,
+            #state{configuration = Config} = State) ->
+    case cloudi_core_i_configuration:services_suspend(L, Config, infinity) of
+        ok ->
+            {reply, ok, State};
+        {error, _} = Error ->
+            {reply, Error, State}
+    end;
+
+handle_call({services_resume, L}, _,
+            #state{configuration = Config} = State) ->
+    case cloudi_core_i_configuration:services_resume(L, Config, infinity) of
+        ok ->
+            {reply, ok, State};
         {error, _} = Error ->
             {reply, Error, State}
     end;
@@ -462,7 +531,7 @@ handle_call(services, _,
     {reply, {ok, cloudi_core_i_configuration:services(Config)}, State};
 
 handle_call({nodes_set, _, _} = Request, _, State) ->
-    nodes_call(Request, State);
+    nodes_set_call(Request, State);
 
 handle_call(nodes_get, _,
             #state{configuration = Config} = State) ->
@@ -955,7 +1024,8 @@ service_start_wait_pid([{MonitorRef, Pid} | M], Error) ->
         {'DOWN', MonitorRef, process, Pid, Info} ->
             ErrorNew = if
                 Error =:= undefined ->
-                    {error, {service_internal_start_failed, Info}};
+                    % first error encountered is returned
+                    {error, Info};
                 true ->
                     Error
             end,
@@ -964,8 +1034,13 @@ service_start_wait_pid([{MonitorRef, Pid} | M], Error) ->
 
 service_start_wait_pids([], undefined, Service) ->
     {ok, Service};
-service_start_wait_pids([], Error, _) ->
-    Error;
+service_start_wait_pids([], {error, Reason}, Service) ->
+    case Service of
+        #config_service_internal{} ->
+            {error, {service_internal_start_failed, Reason}};
+        #config_service_external{} ->
+            {error, {service_external_start_failed, Reason}}
+    end;
 service_start_wait_pids([M | MonitorPids], Error, Service) ->
     service_start_wait_pids(MonitorPids,
                             service_start_wait_pid(M, Error), Service).
@@ -979,8 +1054,6 @@ service_start_wait(Pids, Service) ->
     % and pass the error reason as a return value of the configure/2 function
     % or the cloudi_service_api:services_add/2 function when a service
     % starts for the first time
-    % (restarts require cloudi_core_i_services_monitor to
-    %  call cloudi_core_i_services_monitor:process_init_begin/1)
     ok = cloudi_core_i_services_monitor:process_init_begin(Pids),
     service_start_wait_pids(MonitorPids, undefined, Service).
 
@@ -999,6 +1072,7 @@ service_start_internal(IndexProcess, Pids,
                            dest_list_deny = DestListDeny,
                            dest_list_allow = DestListAllow,
                            options = #config_service_options{
+                               restart_all = RestartAll,
                                restart_delay = RestartDelay,
                                scope = Scope} = Options,
                            max_r = MaxR,
@@ -1013,7 +1087,8 @@ service_start_internal(IndexProcess, Pids,
                   DestRefresh, DestListDeny,
                   DestListAllow, Options, ID],
                  IndexProcess, CountProcess, 1, Scope,
-                 TimeoutTerm, RestartDelay, MaxR, MaxT, ID, Timeout) of
+                 TimeoutTerm, RestartAll, RestartDelay,
+                 MaxR, MaxT, ID, Timeout) of
         {ok, P} ->
             service_format_log(Service, P),
             service_start_internal(IndexProcess + 1, [P | Pids], Service,
@@ -1044,6 +1119,7 @@ service_start_external(IndexProcess, Pids,
                            dest_list_deny = DestListDeny,
                            dest_list_allow = DestListAllow,
                            options = #config_service_options{
+                               restart_all = RestartAll,
                                restart_delay = RestartDelay,
                                scope = Scope} = Options,
                            max_r = MaxR,
@@ -1059,7 +1135,8 @@ service_start_external(IndexProcess, Pids,
                   DestRefresh, DestListDeny,
                   DestListAllow, Options, ID],
                  IndexProcess, CountProcess, CountThread, Scope,
-                 TimeoutTerm, RestartDelay, MaxR, MaxT, ID, Timeout) of
+                 TimeoutTerm, RestartAll, RestartDelay,
+                 MaxR, MaxT, ID, Timeout) of
         {ok, P} ->
             service_format_log(Service, P),
             service_start_external(IndexProcess + 1, [P | Pids], Service,
@@ -1143,6 +1220,42 @@ service_restart_external(#config_service_external{
             {error, {service_external_restart_failed, Reason}}
     end.
 
+service_suspend_internal(#config_service_internal{
+                             uuid = ID}, Timeout) ->
+    case cloudi_core_i_services_monitor:suspend(ID, Timeout) of
+        ok ->
+            ok;
+        {error, Reason} ->
+            {error, {service_internal_suspend_failed, Reason}}
+    end.
+
+service_suspend_external(#config_service_external{
+                             uuid = ID}, Timeout) ->
+    case cloudi_core_i_services_monitor:suspend(ID, Timeout) of
+        ok ->
+            ok;
+        {error, Reason} ->
+            {error, {service_external_suspend_failed, Reason}}
+    end.
+
+service_resume_internal(#config_service_internal{
+                            uuid = ID}, Timeout) ->
+    case cloudi_core_i_services_monitor:resume(ID, Timeout) of
+        ok ->
+            ok;
+        {error, Reason} ->
+            {error, {service_internal_resume_failed, Reason}}
+    end.
+
+service_resume_external(#config_service_external{
+                            uuid = ID}, Timeout) ->
+    case cloudi_core_i_services_monitor:resume(ID, Timeout) of
+        ok ->
+            ok;
+        {error, Reason} ->
+            {error, {service_external_resume_failed, Reason}}
+    end.
+
 service_update_external(CountProcess, [], [], _, _, CountProcess) ->
     ok;
 service_update_external(IndexProcess, Pids, Ports, Arguments,
@@ -1180,27 +1293,30 @@ nodes_call_remote_result_replies([{_, ok} | Replies], Output) ->
 nodes_call_remote_result_replies([{_, _} = Error | Replies], Output) ->
     nodes_call_remote_result_replies(Replies, [Error | Output]).
 
-nodes_call_remote({_, _, remote}, _) ->
+nodes_call_remote({_, _, CallType}, _, _)
+    when CallType =:= remote; CallType =:= local_only ->
     ok;
-nodes_call_remote({F, L, local}, Connect) ->
-    Nodes = if
-        Connect =:= visible ->
-            nodes();
-        Connect =:= hidden ->
-            nodes(connected)
-    end,
-    nodes_call_remote_result(global:trans({{?MODULE, L}, self()}, fun() ->
-        gen_server:multi_call(Nodes, ?MODULE,
-                              {F, L, remote}, infinity)
-    end)).
+nodes_call_remote({F, L, local}, Listen, NodeName) ->
+    case cloudi_x_cpg:listen_nodes(Listen, NodeName) of
+        [] ->
+            ok;
+        [_ | _] = Nodes ->
+            GlobalId = {{?MODULE, L}, self()},
+            nodes_call_remote_result(global:trans(GlobalId, fun() ->
+                gen_server:multi_call(Nodes, ?MODULE,
+                                      {F, L, remote}, infinity)
+            end))
+    end.
 
-nodes_call({F, L, _} = Request,
-           #state{configuration = Config} = State) ->
+nodes_call({F, L, CallType} = Request,
+           #state{node_name = NodeName,
+                  configuration = Config} = State) ->
     case cloudi_core_i_configuration:F(L, Config) of
-        {ok, Config} ->
+        {ok, Config}
+            when CallType =:= remote; CallType =:= local_only ->
             {reply, ok, State};
-        {ok, #config{nodes = #config_nodes{connect = Connect}} = ConfigNew} ->
-            Result = nodes_call_remote(Request, Connect),
+        {ok, #config{nodes = #config_nodes{listen = Listen}} = ConfigNew} ->
+            Result = nodes_call_remote(Request, Listen, NodeName),
             case cloudi_core_i_nodes:reconfigure(ConfigNew, infinity) of
                 ok ->
                     {reply, Result, State#state{configuration = ConfigNew}};
@@ -1211,6 +1327,23 @@ nodes_call({F, L, _} = Request,
             {reply, Error, State}
     end.
 
+nodes_set_call({nodes_set, _, remote} = Request, State) ->
+    nodes_call(Request, State);
+nodes_set_call({nodes_set, L0, local} = Request, State) ->
+    case lists:keytake(set, 1, L0) of
+        false ->
+            nodes_call(Request, State);
+        {value, {set, Value}, LN} ->
+            if
+                Value =:= all ->
+                    nodes_call({nodes_set, LN, local}, State);
+                Value =:= local ->
+                    nodes_call({nodes_set, LN, local_only}, State);
+                true ->
+                    {error, {set_invalid, Value}}
+            end
+    end.
+
 service_terminated_remove([] = Services, _) ->
     Services;
 service_terminated_remove([#config_service_internal{uuid = ID} | Services],
