@@ -9,7 +9,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2011-2019 Michael Truog <mjtruog at protonmail dot com>
+%%% Copyright (c) 2011-2020 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -30,8 +30,8 @@
 %%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog at protonmail dot com>
-%%% @copyright 2011-2019 Michael Truog
-%%% @version 1.8.0 {@date} {@time}
+%%% @copyright 2011-2020 Michael Truog
+%%% @version 1.8.1 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_api).
@@ -45,6 +45,8 @@
          services_add/2,
          services_remove/2,
          services_restart/2,
+         services_suspend/2,
+         services_resume/2,
          services_search/2,
          services_status/2,
          services_update/2,
@@ -89,7 +91,7 @@
 -export_type([priority/0]).
 
 -type dest_refresh_delay_milliseconds() ::
-    (?TIMEOUT_DELTA + 1)..?TIMEOUT_MAX_ERLANG.
+    ?DEST_REFRESH_DELAY_MIN..?DEST_REFRESH_DELAY_MAX.
 -export_type([dest_refresh_delay_milliseconds/0]).
 
 -type timeout_initialize_value_milliseconds() ::
@@ -188,15 +190,15 @@
          Timeout :: timeout_initialize_value_milliseconds(),
          State :: any(),
          Dispatcher :: cloudi_service:dispatcher()) ->
-        {ok, NewState :: any()} |
-        {stop, Reason :: any(), NewState :: any()}).
+        {ok, StateNew :: any()} |
+        {stop, Reason :: any(), StateNew :: any()}).
 -type aspect_init_after_external_f() ::
     fun((CommandLine :: list(string()),
          Prefix :: cloudi:service_name_pattern(),
          Timeout :: timeout_initialize_value_milliseconds(),
          State :: any()) ->
-        {ok, NewState :: any()} |
-        {stop, Reason :: any(), NewState :: any()}).
+        {ok, StateNew :: any()} |
+        {stop, Reason :: any(), StateNew :: any()}).
 -type aspect_init_after_internal() ::
     aspect_init_after_internal_f() |
     {Module :: module(), Function :: atom()} |
@@ -221,8 +223,8 @@
          Source :: cloudi_service:source(),
          State :: any(),
          Dispatcher :: cloudi_service:dispatcher()) ->
-        {ok, NewState :: any()} |
-        {stop, Reason :: any(), NewState :: any()}).
+        {ok, StateNew :: any()} |
+        {stop, Reason :: any(), StateNew :: any()}).
 -type aspect_request_before_external_f() ::
     fun((RequestType :: cloudi_service:request_type(),
          Name :: cloudi_service:service_name(),
@@ -234,8 +236,8 @@
          TransId :: cloudi_service:trans_id(),
          Source :: cloudi_service:source(),
          State :: any()) ->
-        {ok, NewState :: any()} |
-        {stop, Reason :: any(), NewState :: any()}).
+        {ok, StateNew :: any()} |
+        {stop, Reason :: any(), StateNew :: any()}).
 -type aspect_request_after_internal_f() ::
     fun((RequestType :: cloudi_service:request_type(),
          Name :: cloudi_service:service_name(),
@@ -249,8 +251,8 @@
          Result :: cloudi_service:request_result(),
          State :: any(),
          Dispatcher :: cloudi_service:dispatcher()) ->
-        {ok, NewState :: any()} |
-        {stop, Reason :: any(), NewState :: any()}).
+        {ok, StateNew :: any()} |
+        {stop, Reason :: any(), StateNew :: any()}).
 -type aspect_request_after_external_f() ::
     fun((RequestType :: cloudi_service:request_type(),
          Name :: cloudi_service:service_name(),
@@ -263,8 +265,8 @@
          Source :: cloudi_service:source(),
          Result :: cloudi_service:request_result(),
          State :: any()) ->
-        {ok, NewState :: any()} |
-        {stop, Reason :: any(), NewState :: any()}).
+        {ok, StateNew :: any()} |
+        {stop, Reason :: any(), StateNew :: any()}).
 -type aspect_request_before_internal() ::
     aspect_request_before_internal_f() |
     {Module :: module(), Function :: atom()} |
@@ -293,8 +295,8 @@
     fun((Request :: any(),
          State :: any(),
          Dispatcher :: cloudi_service:dispatcher()) ->
-        {ok, NewState :: any()} |
-        {stop, Reason :: any(), NewState :: any()}).
+        {ok, StateNew :: any()} |
+        {stop, Reason :: any(), StateNew :: any()}).
 -type aspect_info_internal() ::
     aspect_info_internal_f() |
     {Module :: module(), Function :: atom()} |
@@ -398,6 +400,7 @@
                {count_min, number()}) | false} |
          {timeout_terminate,
           undefined | timeout_terminate_milliseconds()} |
+         {restart_all, boolean()} |
          {restart_delay,
           list({time_exponential_min, restart_delay_milliseconds()} |
                {time_exponential_max, restart_delay_milliseconds()} |
@@ -486,6 +489,7 @@
                {count_min, number()}) | false} |
          {timeout_terminate,
           undefined | timeout_terminate_milliseconds()} |
+         {restart_all, boolean()} |
          {restart_delay,
           list({time_exponential_min, restart_delay_milliseconds()} |
                {time_exponential_max, restart_delay_milliseconds()} |
@@ -555,8 +559,10 @@
                   {prefix, cloudi:service_name_pattern()} |
                   {module, atom()} |
                   {count_process, pos_integer()} |
+                  {suspended, boolean()} |
                   {uptime_total, nonempty_string()} |
                   {uptime_running, nonempty_string()} |
+                  {uptime_processing, nonempty_string()} |
                   {uptime_restarts, nonempty_string()} |
                   {downtime_day_restarting, nonempty_string()} |
                   {downtime_week_restarting, nonempty_string()} |
@@ -566,26 +572,36 @@
                   {interrupt_week_updating, nonempty_string()} |
                   {interrupt_month_updating, nonempty_string()} |
                   {interrupt_year_updating, nonempty_string()} |
+                  {interrupt_day_suspended, nonempty_string()} |
+                  {interrupt_week_suspended, nonempty_string()} |
+                  {interrupt_month_suspended, nonempty_string()} |
+                  {interrupt_year_suspended, nonempty_string()} |
                   {availability_day_total, nonempty_string()} |
                   {availability_day_running, nonempty_string()} |
                   {availability_day_updated, nonempty_string()} |
+                  {availability_day_processing, nonempty_string()} |
                   {availability_week_total, nonempty_string()} |
                   {availability_week_running, nonempty_string()} |
                   {availability_week_updated, nonempty_string()} |
+                  {availability_week_processing, nonempty_string()} |
                   {availability_month_total, nonempty_string()} |
                   {availability_month_running, nonempty_string()} |
                   {availability_month_updated, nonempty_string()} |
+                  {availability_month_processing, nonempty_string()} |
                   {availability_year_total, nonempty_string()} |
                   {availability_year_running, nonempty_string()} |
-                  {availability_year_updated, nonempty_string()}).
+                  {availability_year_updated, nonempty_string()} |
+                  {availability_year_processing, nonempty_string()}).
 -type service_status_external() ::
     nonempty_list({type, external} |
                   {prefix, cloudi:service_name_pattern()} |
                   {file_path, file:filename()} |
                   {count_process, pos_integer()} |
                   {count_thread, pos_integer()} |
+                  {suspended, boolean()} |
                   {uptime_total, nonempty_string()} |
                   {uptime_running, nonempty_string()} |
+                  {uptime_processing, nonempty_string()} |
                   {uptime_restarts, nonempty_string()} |
                   {downtime_day_restarting, nonempty_string()} |
                   {downtime_week_restarting, nonempty_string()} |
@@ -595,18 +611,26 @@
                   {interrupt_week_updating, nonempty_string()} |
                   {interrupt_month_updating, nonempty_string()} |
                   {interrupt_year_updating, nonempty_string()} |
+                  {interrupt_day_suspended, nonempty_string()} |
+                  {interrupt_week_suspended, nonempty_string()} |
+                  {interrupt_month_suspended, nonempty_string()} |
+                  {interrupt_year_suspended, nonempty_string()} |
                   {availability_day_total, nonempty_string()} |
                   {availability_day_running, nonempty_string()} |
                   {availability_day_updated, nonempty_string()} |
+                  {availability_day_processing, nonempty_string()} |
                   {availability_week_total, nonempty_string()} |
                   {availability_week_running, nonempty_string()} |
                   {availability_week_updated, nonempty_string()} |
+                  {availability_week_processing, nonempty_string()} |
                   {availability_month_total, nonempty_string()} |
                   {availability_month_running, nonempty_string()} |
                   {availability_month_updated, nonempty_string()} |
+                  {availability_month_processing, nonempty_string()} |
                   {availability_year_total, nonempty_string()} |
                   {availability_year_running, nonempty_string()} |
-                  {availability_year_updated, nonempty_string()}).
+                  {availability_year_updated, nonempty_string()} |
+                  {availability_year_processing, nonempty_string()}).
 -type service_status() ::
     service_status_internal() | service_status_external().
 -export_type([service_id/0,
@@ -620,10 +644,10 @@
 
 -type module_version() :: list(any()).
 -type module_state_internal_f() ::
-    fun((OldModuleVersion :: module_version(),
-         NewModuleVersion :: module_version(),
-         OldState :: any()) ->
-        {ok, NewState :: any()} |
+    fun((ModuleVersionOld :: module_version(),
+         ModuleVersionNew :: module_version(),
+         StateOld :: any()) ->
+        {ok, StateNew :: any()} |
         {error, Reason :: any()}).
 -type module_state_internal() ::
     module_state_internal_f() |
@@ -780,27 +804,32 @@
 -type node_reconnect_delay_seconds() ::
     period_seconds().
 -export_type([node_reconnect_delay_seconds/0]).
+-type nodes_properties() ::
+    node() |
+    {nodes, list(node())} |
+    {reconnect_start, node_reconnect_delay_seconds()} |
+    {reconnect_delay, node_reconnect_delay_seconds()} |
+    {listen, visible | all} |
+    {connect, visible | hidden} |
+    {timestamp_type, erlang | os | warp} |
+    {discovery,
+     list({ec2, list({address, inet:ip_address()} |
+                     {port, inet:port_number()} |
+                     {ttl, non_neg_integer()})} |
+          {multicast, list({access_key_id, string()} |
+                           {secret_access_key, string()} |
+                           {ec2_host, string()} |
+                           {groups, list(string())} |
+                           {tags, list({string(), string()} |
+                                       string())})})} |
+    {cost, list({node() | default, float()})} |
+    {cost_precision, 0..253} |
+    {log_reconnect, loglevel()}.
 -type nodes_proplist() ::
-    nonempty_list(node() |
-                  {nodes, list(node())} |
-                  {reconnect_start, node_reconnect_delay_seconds()} |
-                  {reconnect_delay, node_reconnect_delay_seconds()} |
-                  {listen, visible | all} |
-                  {connect, visible | hidden} |
-                  {timestamp_type, erlang | os} |
-                  {discovery,
-                   list({ec2, list({address, inet:ip_address()} |
-                                   {port, inet:port_number()} |
-                                   {ttl, non_neg_integer()})} |
-                        {multicast, list({access_key_id, string()} |
-                                         {secret_access_key, string()} |
-                                         {ec2_host, string()} |
-                                         {groups, list(string())} |
-                                         {tags, list({string(), string()} |
-                                                     string())})})} |
-                  {cost, list({node() | default, float()})} |
-                  {cost_precision, 0..253} |
-                  {log_reconnect, loglevel()}).
+    list(nodes_properties()).
+-type nodes_set_proplist() ::
+    nonempty_list({set, all | local} |
+                  nodes_properties()).
 -type node_status() ::
     nonempty_list(% local node
                   {uptime, nonempty_string()} |
@@ -816,6 +845,7 @@
                   {tracked_cost_week, nonempty_string()} |
                   {tracked_cost_month, nonempty_string()} |
                   {tracked_cost_year, nonempty_string()} |
+                  {connection, visible | hidden} |
                   {tracked_disconnects, nonempty_string()} |
                   {disconnected, boolean()} |
                   {downtime_day_disconnected, nonempty_string()} |
@@ -828,6 +858,7 @@
                   {availability_month, nonempty_string()} |
                   {availability_year, nonempty_string()}).
 -export_type([nodes_proplist/0,
+              nodes_set_proplist/0,
               node_status/0]).
 
 -type aspect_log_f() ::
@@ -956,8 +987,10 @@
 %%%------------------------------------------------------------------------
 
 % timeout for the functions below
+-define(TIMEOUT_SERVICE_API_MIN, ?TIMEOUT_DELTA + 1).
+-define(TIMEOUT_SERVICE_API_MAX, ?TIMEOUT_MAX_ERLANG).
 -type api_timeout_milliseconds() ::
-    (?TIMEOUT_DELTA + 1)..?TIMEOUT_MAX_ERLANG | infinity.
+    ?TIMEOUT_SERVICE_API_MIN..?TIMEOUT_SERVICE_API_MAX | infinity.
 -export_type([api_timeout_milliseconds/0]).
 
 %%-------------------------------------------------------------------------
@@ -981,8 +1014,8 @@
 
 acl_add([_ | _] = L, Timeout)
     when ((is_integer(Timeout) andalso
-           (Timeout > ?TIMEOUT_DELTA) andalso
-           (Timeout =< ?TIMEOUT_MAX_ERLANG)) orelse
+           (Timeout >= ?TIMEOUT_SERVICE_API_MIN) andalso
+           (Timeout =< ?TIMEOUT_SERVICE_API_MAX)) orelse
           (Timeout =:= infinity)) ->
     cloudi_core_i_configurator:acl_add(L, Timeout).
 
@@ -1004,8 +1037,8 @@ acl_add([_ | _] = L, Timeout)
 
 acl_remove([_ | _] = L, Timeout)
     when ((is_integer(Timeout) andalso
-           (Timeout > ?TIMEOUT_DELTA) andalso
-           (Timeout =< ?TIMEOUT_MAX_ERLANG)) orelse
+           (Timeout >= ?TIMEOUT_SERVICE_API_MIN) andalso
+           (Timeout =< ?TIMEOUT_SERVICE_API_MAX)) orelse
           (Timeout =:= infinity)) ->
     cloudi_core_i_configurator:acl_remove(L, Timeout).
 
@@ -1021,8 +1054,8 @@ acl_remove([_ | _] = L, Timeout)
 
 acl(Timeout)
     when ((is_integer(Timeout) andalso
-           (Timeout > ?TIMEOUT_DELTA) andalso
-           (Timeout =< ?TIMEOUT_MAX_ERLANG)) orelse
+           (Timeout >= ?TIMEOUT_SERVICE_API_MIN) andalso
+           (Timeout =< ?TIMEOUT_SERVICE_API_MAX)) orelse
           (Timeout =:= infinity)) ->
     cloudi_core_i_configurator:acl(Timeout).
 
@@ -1046,8 +1079,8 @@ acl(Timeout)
 
 service_subscriptions(ServiceId, Timeout)
     when ((is_integer(Timeout) andalso
-           (Timeout > ?TIMEOUT_DELTA) andalso
-           (Timeout =< ?TIMEOUT_MAX_ERLANG)) orelse
+           (Timeout >= ?TIMEOUT_SERVICE_API_MIN) andalso
+           (Timeout =< ?TIMEOUT_SERVICE_API_MAX)) orelse
           (Timeout =:= infinity)) ->
     case service_id_convert(ServiceId) of
         {ok, ServiceIdValid} ->
@@ -1079,8 +1112,8 @@ service_subscriptions(ServiceId, Timeout)
 
 services_add([_ | _] = L, Timeout)
     when ((is_integer(Timeout) andalso
-           (Timeout > ?TIMEOUT_DELTA) andalso
-           (Timeout =< ?TIMEOUT_MAX_ERLANG)) orelse
+           (Timeout >= ?TIMEOUT_SERVICE_API_MIN) andalso
+           (Timeout =< ?TIMEOUT_SERVICE_API_MAX)) orelse
           (Timeout =:= infinity)) ->
     cloudi_core_i_configurator:services_add(L, Timeout).
 
@@ -1105,8 +1138,8 @@ services_add([_ | _] = L, Timeout)
 
 services_remove([_ | _] = L, Timeout)
     when ((is_integer(Timeout) andalso
-           (Timeout > ?TIMEOUT_DELTA) andalso
-           (Timeout =< ?TIMEOUT_MAX_ERLANG)) orelse
+           (Timeout >= ?TIMEOUT_SERVICE_API_MIN) andalso
+           (Timeout =< ?TIMEOUT_SERVICE_API_MAX)) orelse
           (Timeout =:= infinity)) ->
     case service_ids_convert(L) of
         {ok, ServiceIdsValid} ->
@@ -1139,8 +1172,8 @@ services_remove([_ | _] = L, Timeout)
 
 services_restart([_ | _] = L, Timeout)
     when ((is_integer(Timeout) andalso
-           (Timeout > ?TIMEOUT_DELTA) andalso
-           (Timeout =< ?TIMEOUT_MAX_ERLANG)) orelse
+           (Timeout >= ?TIMEOUT_SERVICE_API_MIN) andalso
+           (Timeout =< ?TIMEOUT_SERVICE_API_MAX)) orelse
           (Timeout =:= infinity)) ->
     case service_ids_convert(L) of
         {ok, ServiceIdsValid} ->
@@ -1150,6 +1183,64 @@ services_restart([_ | _] = L, Timeout)
             Error
     end.
 
+%%-------------------------------------------------------------------------
+%% @doc
+%% ===Suspend service instances.===
+%% Provide the Service UUIDs for the services that should be suspended.
+%% The Service UUID is shown in the output of services/1.
+%% @end
+%%-------------------------------------------------------------------------
+
+-spec services_suspend(L :: nonempty_list(binary() | string()),
+                       Timeout :: api_timeout_milliseconds()) ->
+    ok |
+    {error,
+     timeout | noproc |
+     {service_id_invalid, any()} |
+     cloudi_core_i_configuration:error_reason_services_suspend()}.
+
+services_suspend([_ | _] = L, Timeout)
+    when ((is_integer(Timeout) andalso
+           (Timeout >= ?TIMEOUT_SERVICE_API_MIN) andalso
+           (Timeout =< ?TIMEOUT_SERVICE_API_MAX)) orelse
+          (Timeout =:= infinity)) ->
+    case service_ids_convert(L) of
+        {ok, ServiceIdsValid} ->
+            cloudi_core_i_configurator:services_suspend(ServiceIdsValid,
+                                                        Timeout);
+        {error, _} = Error ->
+            Error
+    end.
+
+%%-------------------------------------------------------------------------
+%% @doc
+%% ===Resume service instances.===
+%% Provide the Service UUIDs for the services that should be resumed.
+%% The Service UUID is shown in the output of services/1.
+%% @end
+%%-------------------------------------------------------------------------
+
+-spec services_resume(L :: nonempty_list(binary() | string()),
+                      Timeout :: api_timeout_milliseconds()) ->
+    ok |
+    {error,
+     timeout | noproc |
+     {service_id_invalid, any()} |
+     cloudi_core_i_configuration:error_reason_services_resume()}.
+
+services_resume([_ | _] = L, Timeout)
+    when ((is_integer(Timeout) andalso
+           (Timeout >= ?TIMEOUT_SERVICE_API_MIN) andalso
+           (Timeout =< ?TIMEOUT_SERVICE_API_MAX)) orelse
+          (Timeout =:= infinity)) ->
+    case service_ids_convert(L) of
+        {ok, ServiceIdsValid} ->
+            cloudi_core_i_configurator:services_resume(ServiceIdsValid,
+                                                       Timeout);
+        {error, _} = Error ->
+            Error
+    end.
+
 %%-------------------------------------------------------------------------
 %% @doc
 %% ===Search service instances for matches on the provided service name.===
@@ -1174,8 +1265,8 @@ services_restart([_ | _] = L, Timeout)
 
 services_search(Name, Timeout)
     when ((is_integer(Timeout) andalso
-           (Timeout > ?TIMEOUT_DELTA) andalso
-           (Timeout =< ?TIMEOUT_MAX_ERLANG)) orelse
+           (Timeout >= ?TIMEOUT_SERVICE_API_MIN) andalso
+           (Timeout =< ?TIMEOUT_SERVICE_API_MAX)) orelse
           (Timeout =:= infinity)) ->
     {Scope, ServiceName} = case Name of
         {ScopeValue, [_ | _] = ServiceNameValue} when is_atom(ScopeValue) ->
@@ -1211,8 +1302,8 @@ services_search(Name, Timeout)
 services_status(L, Timeout)
     when is_list(L),
          ((is_integer(Timeout) andalso
-           (Timeout > ?TIMEOUT_DELTA) andalso
-           (Timeout =< ?TIMEOUT_MAX_ERLANG)) orelse
+           (Timeout >= ?TIMEOUT_SERVICE_API_MIN) andalso
+           (Timeout =< ?TIMEOUT_SERVICE_API_MAX)) orelse
           (Timeout =:= infinity)) ->
     ServiceIdsResult = case service_ids_convert(L) of
         {ok, []} ->
@@ -1251,12 +1342,12 @@ services_status(L, Timeout)
 
 services_update([_ | _] = L, Timeout)
     when ((is_integer(Timeout) andalso
-           (Timeout > ?TIMEOUT_DELTA) andalso
-           (Timeout =< ?TIMEOUT_MAX_ERLANG)) orelse
+           (Timeout >= ?TIMEOUT_SERVICE_API_MIN) andalso
+           (Timeout =< ?TIMEOUT_SERVICE_API_MAX)) orelse
           (Timeout =:= infinity)) ->
     case service_ids_convert_update(L) of
-        {ok, NewL} ->
-            cloudi_core_i_configurator:services_update(NewL, Timeout);
+        {ok, LNew} ->
+            cloudi_core_i_configurator:services_update(LNew, Timeout);
         {error, _} = Error ->
             Error
     end.
@@ -1274,8 +1365,8 @@ services_update([_ | _] = L, Timeout)
 
 services(Timeout)
     when ((is_integer(Timeout) andalso
-           (Timeout > ?TIMEOUT_DELTA) andalso
-           (Timeout =< ?TIMEOUT_MAX_ERLANG)) orelse
+           (Timeout >= ?TIMEOUT_SERVICE_API_MIN) andalso
+           (Timeout =< ?TIMEOUT_SERVICE_API_MAX)) orelse
           (Timeout =:= infinity)) ->
     cloudi_core_i_configurator:services(Timeout).
 
@@ -1285,7 +1376,7 @@ services(Timeout)
 %% @end
 %%-------------------------------------------------------------------------
 
--spec nodes_set(L :: nodes_proplist(),
+-spec nodes_set(L :: nodes_set_proplist(),
                 Timeout :: api_timeout_milliseconds()) ->
     ok |
     {error,
@@ -1294,8 +1385,8 @@ services(Timeout)
 
 nodes_set([_ | _] = L, Timeout)
     when ((is_integer(Timeout) andalso
-           (Timeout > ?TIMEOUT_DELTA) andalso
-           (Timeout =< ?TIMEOUT_MAX_ERLANG)) orelse
+           (Timeout >= ?TIMEOUT_SERVICE_API_MIN) andalso
+           (Timeout =< ?TIMEOUT_SERVICE_API_MAX)) orelse
           (Timeout =:= infinity)) ->
     cloudi_core_i_configurator:nodes_set(L, Timeout).
 
@@ -1311,8 +1402,8 @@ nodes_set([_ | _] = L, Timeout)
 
 nodes_get(Timeout)
     when ((is_integer(Timeout) andalso
-           (Timeout > ?TIMEOUT_DELTA) andalso
-           (Timeout =< ?TIMEOUT_MAX_ERLANG)) orelse
+           (Timeout >= ?TIMEOUT_SERVICE_API_MIN) andalso
+           (Timeout =< ?TIMEOUT_SERVICE_API_MAX)) orelse
           (Timeout =:= infinity)) ->
     cloudi_core_i_configurator:nodes_get(Timeout).
 
@@ -1333,8 +1424,8 @@ nodes_get(Timeout)
 
 nodes_add([_ | _] = L, Timeout)
     when ((is_integer(Timeout) andalso
-           (Timeout > ?TIMEOUT_DELTA) andalso
-           (Timeout =< ?TIMEOUT_MAX_ERLANG)) orelse
+           (Timeout >= ?TIMEOUT_SERVICE_API_MIN) andalso
+           (Timeout =< ?TIMEOUT_SERVICE_API_MAX)) orelse
           (Timeout =:= infinity)) ->
     cloudi_core_i_configurator:nodes_add(L, Timeout).
 
@@ -1354,8 +1445,8 @@ nodes_add([_ | _] = L, Timeout)
 
 nodes_remove([_ | _] = L, Timeout)
     when ((is_integer(Timeout) andalso
-           (Timeout > ?TIMEOUT_DELTA) andalso
-           (Timeout =< ?TIMEOUT_MAX_ERLANG)) orelse
+           (Timeout >= ?TIMEOUT_SERVICE_API_MIN) andalso
+           (Timeout =< ?TIMEOUT_SERVICE_API_MAX)) orelse
           (Timeout =:= infinity)) ->
     cloudi_core_i_configurator:nodes_remove(L, Timeout).
 
@@ -1371,8 +1462,8 @@ nodes_remove([_ | _] = L, Timeout)
 
 nodes_alive(Timeout)
     when ((is_integer(Timeout) andalso
-           (Timeout > ?TIMEOUT_DELTA) andalso
-           (Timeout =< ?TIMEOUT_MAX_ERLANG)) orelse
+           (Timeout >= ?TIMEOUT_SERVICE_API_MIN) andalso
+           (Timeout =< ?TIMEOUT_SERVICE_API_MAX)) orelse
           (Timeout =:= infinity)) ->
     cloudi_core_i_nodes:alive(Timeout).
 
@@ -1388,8 +1479,8 @@ nodes_alive(Timeout)
 
 nodes_dead(Timeout)
     when ((is_integer(Timeout) andalso
-           (Timeout > ?TIMEOUT_DELTA) andalso
-           (Timeout =< ?TIMEOUT_MAX_ERLANG)) orelse
+           (Timeout >= ?TIMEOUT_SERVICE_API_MIN) andalso
+           (Timeout =< ?TIMEOUT_SERVICE_API_MAX)) orelse
           (Timeout =:= infinity)) ->
     cloudi_core_i_nodes:dead(Timeout).
 
@@ -1409,8 +1500,8 @@ nodes_dead(Timeout)
 nodes_status(L, Timeout)
     when is_list(L),
          ((is_integer(Timeout) andalso
-           (Timeout > ?TIMEOUT_DELTA) andalso
-           (Timeout =< ?TIMEOUT_MAX_ERLANG)) orelse
+           (Timeout >= ?TIMEOUT_SERVICE_API_MIN) andalso
+           (Timeout =< ?TIMEOUT_SERVICE_API_MAX)) orelse
           (Timeout =:= infinity)) ->
     cloudi_core_i_nodes:status(L, Timeout).
 
@@ -1426,8 +1517,8 @@ nodes_status(L, Timeout)
 
 nodes(Timeout)
     when ((is_integer(Timeout) andalso
-           (Timeout > ?TIMEOUT_DELTA) andalso
-           (Timeout =< ?TIMEOUT_MAX_ERLANG)) orelse
+           (Timeout >= ?TIMEOUT_SERVICE_API_MIN) andalso
+           (Timeout =< ?TIMEOUT_SERVICE_API_MAX)) orelse
           (Timeout =:= infinity)) ->
     cloudi_core_i_nodes:nodes(Timeout).
 
@@ -1443,8 +1534,8 @@ nodes(Timeout)
 
 logging_set([_ | _] = L, Timeout)
     when ((is_integer(Timeout) andalso
-           (Timeout > ?TIMEOUT_DELTA) andalso
-           (Timeout =< ?TIMEOUT_MAX_ERLANG)) orelse
+           (Timeout >= ?TIMEOUT_SERVICE_API_MIN) andalso
+           (Timeout =< ?TIMEOUT_SERVICE_API_MAX)) orelse
           (Timeout =:= infinity)) ->
     cloudi_core_i_configurator:logging_set(L, Timeout).
 
@@ -1462,8 +1553,8 @@ logging_file_set(FilePath, Timeout)
     when ((FilePath =:= undefined) orelse
           (is_list(FilePath) andalso is_integer(hd(FilePath)))),
          ((is_integer(Timeout) andalso
-           (Timeout > ?TIMEOUT_DELTA) andalso
-           (Timeout =< ?TIMEOUT_MAX_ERLANG)) orelse
+           (Timeout >= ?TIMEOUT_SERVICE_API_MIN) andalso
+           (Timeout =< ?TIMEOUT_SERVICE_API_MAX)) orelse
           (Timeout =:= infinity)) ->
     cloudi_core_i_configurator:logging_file_set(FilePath, Timeout).
 
@@ -1485,8 +1576,8 @@ logging_level_set(Level, Timeout)
           (Level =:= debug) orelse (Level =:= trace) orelse
           (Level =:= off) orelse (Level =:= undefined)),
          ((is_integer(Timeout) andalso
-           (Timeout > ?TIMEOUT_DELTA) andalso
-           (Timeout =< ?TIMEOUT_MAX_ERLANG)) orelse
+           (Timeout >= ?TIMEOUT_SERVICE_API_MIN) andalso
+           (Timeout =< ?TIMEOUT_SERVICE_API_MAX)) orelse
           (Timeout =:= infinity)) ->
     cloudi_core_i_configurator:logging_level_set(Level, Timeout).
 
@@ -1503,8 +1594,8 @@ logging_level_set(Level, Timeout)
 logging_stdout_set(Stdout, Timeout)
     when is_boolean(Stdout),
          ((is_integer(Timeout) andalso
-           (Timeout > ?TIMEOUT_DELTA) andalso
-           (Timeout =< ?TIMEOUT_MAX_ERLANG)) orelse
+           (Timeout >= ?TIMEOUT_SERVICE_API_MIN) andalso
+           (Timeout =< ?TIMEOUT_SERVICE_API_MAX)) orelse
           (Timeout =:= infinity)) ->
     cloudi_core_i_configurator:logging_stdout_set(Stdout, Timeout).
 
@@ -1525,8 +1616,8 @@ logging_stdout_set(Stdout, Timeout)
 logging_syslog_set(L, Timeout)
     when is_list(L) orelse (L =:= undefined),
          ((is_integer(Timeout) andalso
-           (Timeout > ?TIMEOUT_DELTA) andalso
-           (Timeout =< ?TIMEOUT_MAX_ERLANG)) orelse
+           (Timeout >= ?TIMEOUT_SERVICE_API_MIN) andalso
+           (Timeout =< ?TIMEOUT_SERVICE_API_MAX)) orelse
           (Timeout =:= infinity)) ->
     cloudi_core_i_configurator:logging_syslog_set(L, Timeout).
 
@@ -1550,8 +1641,8 @@ logging_syslog_set(L, Timeout)
 logging_formatters_set(L, Timeout)
     when is_list(L) orelse (L =:= undefined),
          ((is_integer(Timeout) andalso
-           (Timeout > ?TIMEOUT_DELTA) andalso
-           (Timeout =< ?TIMEOUT_MAX_ERLANG)) orelse
+           (Timeout >= ?TIMEOUT_SERVICE_API_MIN) andalso
+           (Timeout =< ?TIMEOUT_SERVICE_API_MAX)) orelse
           (Timeout =:= infinity)) ->
     cloudi_core_i_configurator:logging_formatters_set(L, Timeout).
 
@@ -1570,8 +1661,8 @@ logging_formatters_set(L, Timeout)
 logging_redirect_set(Node, Timeout)
     when is_atom(Node),
          ((is_integer(Timeout) andalso
-           (Timeout > ?TIMEOUT_DELTA) andalso
-           (Timeout =< ?TIMEOUT_MAX_ERLANG)) orelse
+           (Timeout >= ?TIMEOUT_SERVICE_API_MIN) andalso
+           (Timeout =< ?TIMEOUT_SERVICE_API_MAX)) orelse
           (Timeout =:= infinity)) ->
     cloudi_core_i_configurator:logging_redirect_set(Node, Timeout).
 
@@ -1587,8 +1678,8 @@ logging_redirect_set(Node, Timeout)
 
 logging(Timeout)
     when ((is_integer(Timeout) andalso
-           (Timeout > ?TIMEOUT_DELTA) andalso
-           (Timeout =< ?TIMEOUT_MAX_ERLANG)) orelse
+           (Timeout >= ?TIMEOUT_SERVICE_API_MIN) andalso
+           (Timeout =< ?TIMEOUT_SERVICE_API_MAX)) orelse
           (Timeout =:= infinity)) ->
     cloudi_core_i_configurator:logging(Timeout).
 
@@ -1607,8 +1698,8 @@ logging(Timeout)
 
 code_path_add(Dir, Timeout)
     when ((is_integer(Timeout) andalso
-           (Timeout > ?TIMEOUT_DELTA) andalso
-           (Timeout =< ?TIMEOUT_MAX_ERLANG)) orelse
+           (Timeout >= ?TIMEOUT_SERVICE_API_MIN) andalso
+           (Timeout =< ?TIMEOUT_SERVICE_API_MAX)) orelse
           (Timeout =:= infinity)) ->
     case code:add_pathz(Dir) of
         true ->
@@ -1632,8 +1723,8 @@ code_path_add(Dir, Timeout)
 
 code_path_remove(Dir, Timeout)
     when ((is_integer(Timeout) andalso
-           (Timeout > ?TIMEOUT_DELTA) andalso
-           (Timeout =< ?TIMEOUT_MAX_ERLANG)) orelse
+           (Timeout >= ?TIMEOUT_SERVICE_API_MIN) andalso
+           (Timeout =< ?TIMEOUT_SERVICE_API_MAX)) orelse
           (Timeout =:= infinity)) ->
     case code:del_path(Dir) of
         true ->
@@ -1656,8 +1747,8 @@ code_path_remove(Dir, Timeout)
 
 code_path(Timeout)
     when ((is_integer(Timeout) andalso
-           (Timeout > ?TIMEOUT_DELTA) andalso
-           (Timeout =< ?TIMEOUT_MAX_ERLANG)) orelse
+           (Timeout >= ?TIMEOUT_SERVICE_API_MIN) andalso
+           (Timeout =< ?TIMEOUT_SERVICE_API_MAX)) orelse
           (Timeout =:= infinity)) ->
     {ok, code:get_path()}.
 
@@ -1678,8 +1769,8 @@ code_path(Timeout)
 
 code_status(Timeout)
     when ((is_integer(Timeout) andalso
-           (Timeout > ?TIMEOUT_DELTA) andalso
-           (Timeout =< ?TIMEOUT_MAX_ERLANG)) orelse
+           (Timeout >= ?TIMEOUT_SERVICE_API_MIN) andalso
+           (Timeout =< ?TIMEOUT_SERVICE_API_MAX)) orelse
           (Timeout =:= infinity)) ->
     TimeNative = cloudi_timestamp:native_monotonic(),
     TimeOffset = erlang:time_offset(),
