@@ -15,11 +15,18 @@
 %%% The Erlang pid is bitwise-XORed from 72 bits down to 32 bits.
 %%% The version 3 (MD5), version 4 (random), and version 5 (SHA)
 %%% methods are provided as specified within the RFC.
+%%%
+%%% The ordered version 1 variant is not present in the RFC,
+%%% though it is the most standards-compliant way of providing
+%%% timestamp UUID ordering.  Timestamp ordering has been used in many
+%%% non-standard UUID formats based on version 1 and typically limited to
+%%% the same data present in the version 1 UUID
+%%% (e.g., "Version 6" at http://gh.peabody.io/uuidv6/).
 %%% @end
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2011-2017 Michael Truog <mjtruog at gmail dot com>
+%%% Copyright (c) 2011-2020 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -39,13 +46,13 @@
 %%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 %%% DEALINGS IN THE SOFTWARE.
 %%%
-%%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2011-2017 Michael Truog
-%%% @version 1.7.1 {@date} {@time}
+%%% @author Michael Truog <mjtruog at protonmail dot com>
+%%% @copyright 2011-2020 Michael Truog
+%%% @version 1.8.0 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(uuid).
--author('mjtruog [at] gmail (dot) com').
+-author('mjtruog at protonmail dot com').
 
 %% external interface
 -export([new/1,
@@ -80,30 +87,44 @@
          test/0]).
 
 -ifdef(ERLANG_OTP_VERSION_16).
--define(TIMESTAMP_ERLANG_NOW, true).
 -else.
 -ifdef(ERLANG_OTP_VERSION_17).
--define(TIMESTAMP_ERLANG_NOW, true).
 -else.
 -define(ERLANG_OTP_VERSION_18_FEATURES, true).
+-ifdef(ERLANG_OTP_VERSION_18).
+-else.
+-ifdef(ERLANG_OTP_VERSION_19).
+-else.
+-define(ERLANG_OTP_VERSION_20_FEATURES, true).
+-ifdef(ERLANG_OTP_VERSION_20).
+-else.
+-ifdef(OTP_RELEASE). % Erlang/OTP >= 21.0
+% able to use -elif here
+-else.
+-error("Erlang/OTP version invalid").
+-endif.
+-endif.
+-endif.
+-endif.
 -endif.
 -endif.
 
--ifdef(TIMESTAMP_ERLANG_NOW).
--type timestamp_type_internal() :: 'erlang_now' | 'os'.
--else.
+-ifdef(ERLANG_OTP_VERSION_18_FEATURES).
 -type timestamp_type_internal() :: 'erlang_timestamp' | 'os' | 'warp'.
+-else.
+-type timestamp_type_internal() :: 'erlang_now' | 'os'.
 -endif.
+-type v1_variant() :: 'rfc4122' | 'ordered'.
 
 -record(uuid_state,
     {
+        variant :: v1_variant(),
         node_id :: <<_:48>>,
         clock_seq :: 0..16383,
         timestamp_type :: timestamp_type_internal(),
         timestamp_last :: integer() % microseconds
     }).
 
-
 -type uuid() :: <<_:128>>.
 -type timestamp_type() :: 'erlang' | 'os' | 'warp'.
 -type state() :: #uuid_state{}.
@@ -113,6 +134,16 @@
 
 -include("uuid.hrl").
 
+% Erlang Binary Term Format constants
+% info from http://erlang.org/doc/apps/erts/erl_ext_dist.html
+-define(TAG_VERSION, 131).
+-define(TAG_PID_EXT, 103).
+-define(TAG_NEW_PID_EXT, 88).
+
+% 16#01b21dd213814000 is the number of 100-ns intervals between the
+% UUID epoch 1582-10-15 00:00:00 and the UNIX epoch 1970-01-01 00:00:00.
+-define(GREGORIAN_EPOCH_OFFSET, 16#01b21dd213814000).
+
 %%%------------------------------------------------------------------------
 %%% External interface functions
 %%%------------------------------------------------------------------------
@@ -144,7 +175,8 @@ new(Pid) when is_pid(Pid) ->
 -spec new(Pid :: pid(),
           Options :: timestamp_type() |
                      list({timestamp_type, timestamp_type()} |
-                          {mac_address, list(non_neg_integer())})) ->
+                          {mac_address, list(non_neg_integer())} |
+                          {variant, v1_variant()})) ->
     state().
 
 new(Pid, TimestampType)
@@ -167,54 +199,58 @@ new(Pid, Options)
         false ->
             mac_address()
     end,
+    Variant = case lists:keyfind(variant, 1, Options) of
+        {variant, Value3} ->
+            Value3;
+        false ->
+            rfc4122
+    end,
 
     % make the version 1 UUID specific to the Erlang node and pid
 
     % 48 bits for the first MAC address found is included with the
-    % distributed Erlang node name
-    <<NodeD01, NodeD02, NodeD03, NodeD04, NodeD05,
-      NodeD06, NodeD07, NodeD08, NodeD09, NodeD10,
-      NodeD11, NodeD12, NodeD13, NodeD14, NodeD15,
-      NodeD16, NodeD17, NodeD18, NodeD19, NodeD20>> =
-      crypto:hash(sha, erlang:list_to_binary(MacAddress ++
-                                             erlang:atom_to_list(node()))),
-    % later, when the pid format changes, handle the different format
-    ExternalTermFormatVersion = 131,
-    PidExtType = 103,
-    <<ExternalTermFormatVersion:8,
-      PidExtType:8,
-      PidBin/binary>> = erlang:term_to_binary(Pid),
-    % 72 bits for the Erlang pid
-    <<PidID1:8, PidID2:8, PidID3:8, PidID4:8, % ID (Node specific, 15 bits)
-      PidSR1:8, PidSR2:8, PidSR3:8, PidSR4:8, % Serial (extra uniqueness)
-      PidCR1:8                       % Node Creation Count
-      >> = binary:part(PidBin, erlang:byte_size(PidBin), -9),
-    % reduce the 160 bit NodeData checksum to 16 bits
-    NodeByte1 = ((((((((NodeD01 bxor NodeD02)
-                       bxor NodeD03)
-                      bxor NodeD04)
-                     bxor NodeD05)
-                    bxor NodeD06)
-                   bxor NodeD07)
-                  bxor NodeD08)
-                 bxor NodeD09)
-                bxor NodeD10,
-    NodeByte2 = (((((((((NodeD11 bxor NodeD12)
-                        bxor NodeD13)
-                       bxor NodeD14)
-                      bxor NodeD15)
-                     bxor NodeD16)
-                    bxor NodeD17)
-                   bxor NodeD18)
-                  bxor NodeD19)
-                 bxor NodeD20)
-                bxor PidCR1,
-    % reduce the Erlang pid to 32 bits
-    PidByte1 = PidID1 bxor PidSR4,
-    PidByte2 = PidID2 bxor PidSR3,
-    PidByte3 = PidID3 bxor PidSR2,
-    PidByte4 = PidID4 bxor PidSR1,
-    ClockSeq = pseudo_random(16384) - 1,
+    % distributed Erlang node name to be hashed with the node creation count
+    NodeData = [MacAddress, erlang:atom_to_binary(node(), utf8)],
+
+    % Reduce the node information to 16 bits and the pid information to 32 bits
+    % for use as the UUID v1 Node Id
+    {NodeCreationCount, PidData} = case erlang:term_to_binary(Pid) of
+        % (when the pid format changes, handle the different formats)
+        <<?TAG_VERSION:8,
+          ?TAG_PID_EXT:8,PidBin/binary>> ->
+            % 72 bits for the Erlang pid
+            <<PidID1:8,PidID2:8,PidID3:8,PidID4:8,% ID (Node specific, 15 bits)
+              PidSR1:8,PidSR2:8,PidSR3:8,PidSR4:8,% Serial (extra uniqueness)
+              PidCR:8                             % Node Creation Count
+              >> = binary:part(PidBin, erlang:byte_size(PidBin), -9),
+            {PidCR,
+             [PidID1, PidID2, PidID3, PidID4,
+              PidSR1, PidSR2, PidSR3, PidSR4]};
+        % format supported in Erlang/OTP 19.0-rc1
+        % required for Erlang/OTP 23.0 (and Erlang/OTP 22.0-rc2)
+        <<?TAG_VERSION:8,
+          ?TAG_NEW_PID_EXT:8,PidBin/binary>> ->
+            % 96 bits for the Erlang pid
+            <<PidID1:8,PidID2:8,PidID3:8,PidID4:8,% ID (Node specific, 15 bits)
+              PidSR1:8,PidSR2:8,PidSR3:8,PidSR4:8,% Serial (extra uniqueness)
+              PidCR:32                            % Node Creation Count
+              >> = binary:part(PidBin, erlang:byte_size(PidBin), -12),
+            {PidCR,
+             [PidID1, PidID2, PidID3, PidID4,
+              PidSR1, PidSR2, PidSR3, PidSR4]}
+    end,
+    Node32 = quickrand_hash:jenkins_32(NodeData, NodeCreationCount),
+    Node16 = (Node32 bsr 16) bxor (Node32 band 16#FFFF),
+    Pid32 = quickrand_hash:jenkins_32(PidData),
+    NodeId = <<Node16:16/big-unsigned-integer,
+               Pid32:32/big-unsigned-integer>>,
+
+    ClockSeq = if
+        Variant =:= rfc4122 ->
+            pseudo_random(16384) - 1;
+        Variant =:= ordered ->
+            pseudo_random(8192) - 1
+    end,
     TimestampTypeInternal = if
         TimestampType =:= os ->
             os;
@@ -231,9 +267,8 @@ new(Pid, Options)
             end
     end,
     TimestampLast = timestamp(TimestampTypeInternal),
-    #uuid_state{node_id = <<NodeByte1:8, NodeByte2:8,
-                            PidByte1:8, PidByte2:8,
-                            PidByte3:8, PidByte4:8>>,
+    #uuid_state{variant = Variant,
+                node_id = NodeId,
                 clock_seq = ClockSeq,
                 timestamp_type = TimestampTypeInternal,
                 timestamp_last = TimestampLast}.
@@ -245,16 +280,15 @@ new(Pid, Options)
 %%-------------------------------------------------------------------------
 
 -spec get_v1(State :: state()) ->
-    {uuid(), NewState :: state()}.
+    {uuid(), StateNew :: state()}.
 
-get_v1(#uuid_state{node_id = NodeId,
+get_v1(#uuid_state{variant = rfc4122,
+                   node_id = NodeId,
                    clock_seq = ClockSeq,
                    timestamp_type = TimestampTypeInternal,
                    timestamp_last = TimestampLast} = State) ->
     MicroSeconds = timestamp(TimestampTypeInternal, TimestampLast),
-    % 16#01b21dd213814000 is the number of 100-ns intervals between the
-    % UUID epoch 1582-10-15 00:00:00 and the UNIX epoch 1970-01-01 00:00:00.
-    Time = MicroSeconds * 10 + 16#01b21dd213814000,
+    Time = MicroSeconds * 10 + ?GREGORIAN_EPOCH_OFFSET,
     % will be larger than 60 bits after 5236-03-31 21:21:00
     <<TimeHigh:12, TimeMid:16, TimeLow:32>> = <<Time:60>>,
     {<<TimeLow:32, TimeMid:16,
@@ -263,6 +297,22 @@ get_v1(#uuid_state{node_id = NodeId,
        1:1, 0:1,            % RFC 4122 variant bits
        ClockSeq:14,
        NodeId/binary>>,
+     State#uuid_state{timestamp_last = MicroSeconds}};
+get_v1(#uuid_state{variant = ordered,
+                   node_id = NodeId,
+                   clock_seq = ClockSeq,
+                   timestamp_type = TimestampTypeInternal,
+                   timestamp_last = TimestampLast} = State) ->
+    MicroSeconds = timestamp(TimestampTypeInternal, TimestampLast),
+    Time = MicroSeconds * 10 + ?GREGORIAN_EPOCH_OFFSET,
+    % will be larger than 60 bits after 5236-03-31 21:21:00
+    <<TimeHigh:48, TimeLow:12>> = <<Time:60>>,
+    {<<TimeHigh:48,
+       0:1, 0:1, 0:1, 1:1,  % version 1 bits
+       TimeLow:12,
+       1:1, 1:1, 1:1,       % ordered (future definition) variant bits
+       ClockSeq:13,
+       NodeId/binary>>,
      State#uuid_state{timestamp_last = MicroSeconds}}.
 
 %%-------------------------------------------------------------------------
@@ -288,32 +338,7 @@ get_v1_time() ->
 -spec get_v1_time(timestamp_type() | state() | uuid()) ->
     non_neg_integer().
 
--ifdef(TIMESTAMP_ERLANG_NOW).
-% Erlang < 18.0
-get_v1_time(erlang) ->
-    timestamp(erlang_now);
-
-get_v1_time(os) ->
-    timestamp(os);
-
-get_v1_time(warp) ->
-    erlang:exit(badarg);
-
-get_v1_time(#uuid_state{timestamp_type = TimestampTypeInternal}) ->
-    timestamp(TimestampTypeInternal);
-
-get_v1_time(Value)
-    when is_binary(Value), byte_size(Value) == 16 ->
-    <<TimeLow:32, TimeMid:16,
-      0:1, 0:1, 0:1, 1:1,  % version 1 bits
-      TimeHigh:12,
-      1:1, 0:1,            % RFC 4122 variant bits
-      _:14,
-      _:48>> = Value,
-    <<Time:60>> = <<TimeHigh:12, TimeMid:16, TimeLow:32>>,
-    ((Time - 16#01b21dd213814000) div 10). % microseconds since UNIX epoch
--else.
-% Erlang >= 18.0
+-ifdef(ERLANG_OTP_VERSION_18_FEATURES).
 get_v1_time(erlang) ->
     timestamp(erlang_timestamp);
 
@@ -328,14 +353,55 @@ get_v1_time(#uuid_state{timestamp_type = TimestampTypeInternal}) ->
 
 get_v1_time(Value)
     when is_binary(Value), byte_size(Value) == 16 ->
-    <<TimeLow:32, TimeMid:16,
-      0:1, 0:1, 0:1, 1:1,  % version 1 bits
-      TimeHigh:12,
-      1:1, 0:1,            % RFC 4122 variant bits
-      _:14,
-      _:48>> = Value,
-    <<Time:60>> = <<TimeHigh:12, TimeMid:16, TimeLow:32>>,
-    ((Time - 16#01b21dd213814000) div 10). % microseconds since UNIX epoch
+    <<Time:60>> = case Value of
+        <<TimeLow:32, TimeMid:16,
+          0:1, 0:1, 0:1, 1:1,  % version 1 bits
+          TimeHigh:12,
+          1:1, 0:1,            % RFC 4122 variant bits
+          _:14,
+          _:48>> ->
+            <<TimeHigh:12, TimeMid:16, TimeLow:32>>;
+        <<TimeHigh:48,
+          0:1, 0:1, 0:1, 1:1,  % version 1 bits
+          TimeLow:12,
+          1:1, 1:1, 1:1,       % ordered (future definition) variant bits
+          _:13,
+          _:48>> ->
+            <<TimeHigh:48, TimeLow:12>>
+    end,
+    (Time - ?GREGORIAN_EPOCH_OFFSET) div 10. % microseconds since UNIX epoch
+-else.
+get_v1_time(erlang) ->
+    timestamp(erlang_now);
+
+get_v1_time(os) ->
+    timestamp(os);
+
+get_v1_time(warp) ->
+    erlang:exit(badarg);
+
+get_v1_time(#uuid_state{timestamp_type = TimestampTypeInternal}) ->
+    timestamp(TimestampTypeInternal);
+
+get_v1_time(Value)
+    when is_binary(Value), byte_size(Value) == 16 ->
+    <<Time:60>> = case Value of
+        <<TimeLow:32, TimeMid:16,
+          0:1, 0:1, 0:1, 1:1,  % version 1 bits
+          TimeHigh:12,
+          1:1, 0:1,            % RFC 4122 variant bits
+          _:14,
+          _:48>> ->
+            <<TimeHigh:12, TimeMid:16, TimeLow:32>>;
+        <<TimeHigh:48,
+          0:1, 0:1, 0:1, 1:1,  % version 1 bits
+          TimeLow:12,
+          1:1, 1:1, 1:1,       % ordered (future definition) variant bits
+          _:13,
+          _:48>> ->
+            <<TimeHigh:48, TimeLow:12>>
+    end,
+    (Time - ?GREGORIAN_EPOCH_OFFSET) div 10. % microseconds since UNIX epoch
 -endif.
 
 %%-------------------------------------------------------------------------
@@ -407,6 +473,13 @@ is_v1(<<_TimeLow:32, _TimeMid:16,
         _ClockSeq:14,
         _NodeId:48>>) ->
     true;
+is_v1(<<_TimeHigh:48,
+        0:1, 0:1, 0:1, 1:1,  % version 1 bits
+        _TimeLow:12,
+        1:1, 1:1, 1:1,       % ordered (future definition) variant bits
+        _ClockSeq:13,
+        _NodeId:48>>) ->
+    true;
 is_v1(_) ->
     false.
 
@@ -559,13 +632,13 @@ get_v4(cached) ->
       1:1, 0:1,            % RFC 4122 variant bits
       Rand3:62>>;
 get_v4(Cache) when element(1, Cache) =:= quickrand_cache ->
-    {<<Rand1:48, _:4, Rand2:12, _:2, Rand3:62>>, NewCache} =
+    {<<Rand1:48, _:4, Rand2:12, _:2, Rand3:62>>, CacheNew} =
         quickrand_cache:rand_bytes(16, Cache),
     {<<Rand1:48,
        0:1, 1:1, 0:1, 0:1,  % version 4 bits
        Rand2:12,
        1:1, 0:1,            % RFC 4122 variant bits
-       Rand3:62>>, NewCache}.
+       Rand3:62>>, CacheNew}.
 
 %%-------------------------------------------------------------------------
 %% @doc
@@ -911,6 +984,8 @@ string_to_uuid(N01, N02, N03, N04, N05, N06, N07, N08,
 -spec is_uuid(any()) ->
     boolean().
 
+is_uuid(<<0:128>>) ->
+    true;
 is_uuid(<<_:48,
           Version:4/unsigned-integer,
           _:12,
@@ -920,6 +995,12 @@ is_uuid(<<_:48,
     (Version == 3) orelse
     (Version == 4) orelse
     (Version == 5);
+is_uuid(<<_:48,
+          Version:4/unsigned-integer,
+          _:12,
+          1:1, 1:1, 1:1,       % ordered (future definition) variant bits
+          _:61>>) ->
+    (Version == 1);
 is_uuid(_) ->
     false.
 
@@ -942,18 +1023,37 @@ increment(<<TimeLow:32, TimeMid:16,
             1:1, 0:1,            % RFC 4122 variant bits
             ClockSeq:14,
             NodeId:48>>) ->
-    NextClockSeq = ClockSeq + 1,
-    NewClockSeq = if
-        NextClockSeq == 16384 ->
+    ClockSeqNext = ClockSeq + 1,
+    ClockSeqNew = if
+        ClockSeqNext == 16384 ->
             0;
         true ->
-            NextClockSeq
+            ClockSeqNext
     end,
     <<TimeLow:32, TimeMid:16,
       0:1, 0:1, 0:1, 1:1,  % version 1 bits
       TimeHigh:12,
       1:1, 0:1,            % RFC 4122 variant bits
-      NewClockSeq:14,
+      ClockSeqNew:14,
+      NodeId:48>>;
+increment(<<TimeHigh:48,
+            0:1, 0:1, 0:1, 1:1,  % version 1 bits
+            TimeLow:12,
+            1:1, 1:1, 1:1,       % ordered (future definition) variant bits
+            ClockSeq:13,
+            NodeId:48>>) ->
+    ClockSeqNext = ClockSeq + 1,
+    ClockSeqNew = if
+        ClockSeqNext == 8192 ->
+            0;
+        true ->
+            ClockSeqNext
+    end,
+    <<TimeHigh:48,
+      0:1, 0:1, 0:1, 1:1,  % version 1 bits
+      TimeLow:12,
+      1:1, 1:1, 1:1,       % ordered (future definition) variant bits
+      ClockSeqNew:13,
       NodeId:48>>;
 increment(<<Rand1:48,
             Version:4/unsigned-integer,
@@ -965,31 +1065,33 @@ increment(<<Rand1:48,
                                                     Rand2:12,
                                                     Rand3:62,
                                                     0:6>>,
-    NextValue = Value + 1,
-    NewValue = if
-        NextValue == 5316911983139663491615228241121378304 ->
+    ValueNext = Value + 1,
+    ValueNew = if
+        ValueNext == 5316911983139663491615228241121378304 ->
             1;
         true ->
-            NextValue
+            ValueNext
     end,
-    <<NewRand1:48,
-      NewRand2:12,
-      NewRand3:62,
-      0:6>> = <<NewValue:16/little-unsigned-integer-unit:8>>,
-    <<NewRand1:48,
+    <<Rand1New:48,
+      Rand2New:12,
+      Rand3New:62,
+      0:6>> = <<ValueNew:16/little-unsigned-integer-unit:8>>,
+    <<Rand1New:48,
       Version:4/unsigned-integer,
-      NewRand2:12,
+      Rand2New:12,
       1:1, 0:1,            % RFC 4122 variant bits
-      NewRand3:62>>;
-increment(#uuid_state{clock_seq = ClockSeq} = State) ->
-    NextClockSeq = ClockSeq + 1,
-    NewClockSeq = if
-        NextClockSeq == 16384 ->
+      Rand3New:62>>;
+increment(#uuid_state{variant = Variant,
+                      clock_seq = ClockSeq} = State) ->
+    ClockSeqNext = ClockSeq + 1,
+    ClockSeqNew = if
+        (Variant =:= rfc4122) andalso (ClockSeqNext == 16384);
+        (Variant =:= ordered) andalso (ClockSeqNext == 8192) ->
             0;
         true ->
-            NextClockSeq
+            ClockSeqNext
     end,
-    State#uuid_state{clock_seq = NewClockSeq}.
+    State#uuid_state{clock_seq = ClockSeqNew}.
 
 %%-------------------------------------------------------------------------
 %% @doc
@@ -1014,6 +1116,7 @@ mac_address() ->
     ok.
 
 test() ->
+    true = uuid:is_uuid(<<0:128>>),
     % version 1 tests
     % uuidgen -t ; date
     % "Fri Dec  7 19:13:58 PST 2012"
@@ -1110,6 +1213,28 @@ test() ->
     V1uuidL1 = [V1uuid11, V1uuid9, V1uuid8, V1uuid7,
                 V1uuid6, V1uuid10, V1uuid5],
     true = V1uuidL0 == lists:usort(V1uuidL1),
+    % version 1 ordered variant
+    V1uuidOrdered0 = uuid:string_to_uuid("1ea55423c2f912f6e847e7a9663898ef"),
+    V1uuidOrdered1 = uuid:string_to_uuid("1ea554248aa31464e847e7a9663898ef"),
+    true = V1uuidOrdered1 > V1uuidOrdered0,
+    true = uuid:is_v1(V1uuidOrdered0),
+    true = uuid:is_v1(V1uuidOrdered1),
+    V1uuidOrdered2 = uuid:increment(V1uuidOrdered1),
+    true = uuid:is_v1(V1uuidOrdered2),
+    <<_:48,
+      0:1, 0:1, 0:1, 1:1,  % version 1 bits
+      _:12,
+      1:1, 1:1, 1:1,       % ordered (future definition) variant bits
+      _:13,
+      _:48>> = V1uuidOrdered2,
+    V1OrderedState0 = uuid:new(self(), [{variant, ordered}]),
+    {V1uuidOrdered3, _} = uuid:get_v1(V1OrderedState0),
+    <<_:48,
+      0:1, 0:1, 0:1, 1:1,  % version 1 bits
+      _:12,
+      1:1, 1:1, 1:1,       % ordered (future definition) variant bits
+      _:13,
+      _:48>> = V1uuidOrdered3,
 
     % version 3 tests
     % $ python
@@ -1251,8 +1376,41 @@ test() ->
            {int_to_hex,1},
            {hex_to_int,1}]}).
 
--ifdef(TIMESTAMP_ERLANG_NOW).
+-ifdef(ERLANG_OTP_VERSION_18_FEATURES).
+timestamp_type_erlang() ->
+    % Erlang >= 18.0
+    true = erlang:function_exported(erlang, system_time, 0),
+    erlang_timestamp.
 
+-ifdef(ERLANG_OTP_VERSION_20_FEATURES).
+timestamp(erlang_timestamp) ->
+    erlang:system_time(microsecond);
+timestamp(os) ->
+    os:system_time(microsecond);
+timestamp(warp) ->
+    erlang:system_time(microsecond).
+-else.
+timestamp(erlang_timestamp) ->
+    erlang:system_time(micro_seconds);
+timestamp(os) ->
+    os:system_time(micro_seconds);
+timestamp(warp) ->
+    erlang:system_time(micro_seconds).
+-endif.
+
+timestamp(erlang_timestamp, TimestampLast) ->
+    TimestampNext = timestamp(erlang_timestamp),
+    if
+        TimestampNext > TimestampLast ->
+            TimestampNext;
+        true ->
+            TimestampLast + 1
+    end;
+timestamp(os, _) ->
+    timestamp(os);
+timestamp(warp, _) ->
+    timestamp(warp).
+-else.
 timestamp_type_erlang() ->
     % Erlang < 18.0
     false = erlang:function_exported(erlang, system_time, 0),
@@ -1269,35 +1427,6 @@ timestamp(erlang_now, _) ->
     timestamp(erlang_now);
 timestamp(os, _) ->
     timestamp(os).
-
--else.
-
-timestamp_type_erlang() ->
-    % Erlang >= 18.0
-    true = erlang:function_exported(erlang, system_time, 0),
-    erlang_timestamp.
-
-timestamp(erlang_timestamp) ->
-    erlang:system_time(micro_seconds);
-timestamp(os) ->
-    {MegaSeconds, Seconds, MicroSeconds} = os:timestamp(),
-    (MegaSeconds * 1000000 + Seconds) * 1000000 + MicroSeconds;
-timestamp(warp) ->
-    erlang:system_time(micro_seconds).
-
-timestamp(erlang_timestamp, TimestampLast) ->
-    TimestampNext = timestamp(erlang_timestamp),
-    if
-        TimestampNext > TimestampLast ->
-            TimestampNext;
-        true ->
-            TimestampLast + 1
-    end;
-timestamp(os, _) ->
-    timestamp(os);
-timestamp(warp, _) ->
-    timestamp(warp).
-
 -endif.
 
 int_to_dec_list(I, N) when is_integer(I), I >= 0 ->
@@ -1359,7 +1488,7 @@ mac_address([{_, L} | Rest]) ->
 
 -ifdef(ERLANG_OTP_VERSION_18_FEATURES).
 pseudo_random(N) ->
-    % assuming exsplus for 58 bits, period 8.31e34
+    % assuming exsp/exsplus for 58 bits, period 8.31e34
     rand:uniform(N).
 -else.
 pseudo_random(N) ->
