@@ -3,7 +3,7 @@
 //
 // MIT License
 //
-// Copyright (c) 2014-2017 Michael Truog <mjtruog at protonmail dot com>
+// Copyright (c) 2014-2020 Michael Truog <mjtruog at protonmail dot com>
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -57,7 +57,7 @@ Task.prototype.request = function (request_type, name, pattern,
                                    request_info, request,
                                    timeout, priority, trans_id, pid) {
     var Task = this;
-    var http_qs = Task._api.info_key_value_parse(request);
+    var http_qs = CloudI.API.info_key_value_parse(request);
     var value = http_qs['value'];
     var response;
     if (value === undefined) {
@@ -71,8 +71,11 @@ Task.prototype.request = function (request_type, name, pattern,
         response =
 '<http_test><value>' + value + '</value></http_test>';
     }
+    var http_response_headers = {};
+    http_response_headers['content-type'] = 'text/xml; charset=utf-8';
+    var response_info = CloudI.API.info_key_value_new(http_response_headers);
     Task._api.return_(request_type, name, pattern,
-                      '', response, timeout, trans_id, pid);
+                      response_info, response, timeout, trans_id, pid);
 };
 
 assert(CloudI.API.thread_count() == 1);
